/*
 * remote.cc: remote contol application for DVBs PCI card
 *
 * Copyright (C) 2000 Matjaz Thaler
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * Or, point your browser to http://www.gnu.org/copyleft/gpl.html
 * 
 * The author can be reached at matjaz.thaler@rd.iskraemeco.si 
 *
 * the project's page is at http://www2.arnes.si/~mthale1/index.html
 */

#include <stdio.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <errno.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <fcntl.h>
#include <sys/time.h>
#include <signal.h>
#include "channel.h"
#include "lirc_client.h"
#include "OSD.h"
#include "remote.h"

void remoteControl();
void main_();
void showOSD(char *text, int dev);
void closeOSD(int dev);
void channel_set(Channel &ch, char *buf,int dev);
void rotateDish(int num);
void drawOSD(char sim, int dev);
int getFileName(int dev, char *fileName, char *dir);
int next_file(char *file_name, int *file, int next_file);
int file_seek(int dev, char *file_name, int *file, int go_to);
int go_to(int dev, int direction);
int readSatList(struct satelliteList *satList);

int shmid;
struct ir_command  *ir;
int childpid;

int main (int argc, char *argv[]){

  shmid = shmget(IPC_PRIVATE,  sizeof(struct ir_command), 0600);
  if(shmid  == -1){
    fprintf(stderr,"ERROR: shmget failed, errno is"); perror(" ");
    exit(-1);
  }
  ir = (struct ir_command*) shmat(shmid, 0, 0);
  shmctl(shmid, IPC_RMID, 0);
  ir->state = EMPTY;
  childpid=fork();
  if(!childpid){
    remoteControl();
    exit(0);
  }
  main_();
  return(0);  
}

void remoteControl(){
  char *buf;
  char remote[20];

  if(lirc_init("remote", 0)==-1){ 
    fprintf(stderr, "lirc: Connection refused\n");
    exit(EXIT_FAILURE);  
  }
  while(lirc_nextcode(&buf)!=-1){
    while(ir->state != EMPTY)
      usleep(50000);
    sscanf(buf, "%*s %*s %s %s", ir->command, remote);
    //printf("lirc: remote: %s command: %s\n", remote, ir->command);
    if(!strcmp(remote, DIGITAL)){
      if(ir->command[1]==0x0 && ir->command[0]>0x2f && ir->command[0]<0x3a){
	ir->number=ir->command[0]-0x30;
	ir->state = NUM_KEY;
      }
      else
	ir->state = COMMAND;
    }
  }
}

void main_(){
  ChannelList chlist;
  int channel=0;
  int channelPrev=0;
  int videoDev;
  int timeOSD=0;
  int timeChann=0;
  int changeChannel=0;
  char tmpBuff[100];
  char *buff=tmpBuff;
  int i,j;
  int irLoop=0;
  char chanBuf[80];
  char *channelName=chanBuf;
  struct satelliteList satList;
  struct video_play_mode pmode;
  struct timeval tv;
  struct timezone tz;
  tm *timef;
  char fileName[128];
  char directory[96];
  FILE *outFile;
  int file;
  char videoBuff[2048];

  strcpy(directory, "/video");
  if(!readSatList(&satList)){
    kill(childpid, 15);
    exit(1);
  }
  videoDev = open("/dev/video", O_RDWR);
  if(videoDev==-1){
    fprintf(stderr, "could not open /dev/video\n");
    kill(childpid, 15);
    exit(1);
  }
  showOSD(satList.sat[satList.current].name, videoDev);
  chlist.init(satList.sat[satList.current].file);
  printf("Reading channel list from %s\n", satList.sat[satList.current].file);
  //rotateDish(satList.sat[satList.current].num);
  //sleep(2);
  channel_set(chlist(channel), channelName, videoDev);
  showOSD(channelName, videoDev);
  timeOSD=30;
  while(1){
    while(ir->state == EMPTY){
      if(timeOSD > 0){
	if(--timeOSD == 0)
	  closeOSD(videoDev);
      }
      if(timeChann < 100)
	timeChann++;
      usleep(100000);
    }
    
    if(ir->state == NUM_KEY){
      if(timeChann<12 && channel<100 && (10*channel+ir->number<chlist.NumChan()))
	channel = 10*channel+ir->number;
      else{
	channelPrev = channel;
	channel = ir->number;
      }	  
      changeChannel = 1;
      ir->state=EMPTY;
    }
    
    if(!strcmp(ir->command, PREVIEW)){
      ir->state=EMPTY;
      i=3;
      sprintf(buff, " set preview time 1 .. 9 sec");
      showOSD(buff, videoDev);
      sleep(3);
      while(ir->state != COMMAND){
	channelPrev=channel;
	channel++;
	if(channel >= chlist.NumChan()-1)
	  channel=0;
	channel_set(chlist(channel), channelName, videoDev);
	showOSD(channelName, videoDev);
	j=i+1;
	while(j>0){
	  j--;
	  sleep(1);
	  if(ir->state==NUM_KEY){
	    if(ir->number==0)
	      j=0;
	    else{
	      i=ir->number;
	      j=i;
	      sprintf(buff, " preview time is %d sec" ,i);
	      showOSD(buff, videoDev);
	    }
	    ir->state=EMPTY;
	  }
	  if(ir->state==COMMAND)
	    j=0;
	}
      }
      ir->state=EMPTY;
      showOSD(channelName, videoDev);
      timeOSD=30;
    }

    if(!strcmp(ir->command, UP)){
      ir->state=EMPTY;
      channelPrev=channel;
      channel++;
      if(channel >= chlist.NumChan())
	channel=0;
      changeChannel = 1;
    }

    if(!strcmp(ir->command, DOWN)){
      ir->state=EMPTY; 
      channelPrev=channel;
      channel--;
      if(channel < 0)
	channel=chlist.NumChan()-1;
      changeChannel = 1;
    }

    if(!strcmp(ir->command, LOOP)){
      ir->state=EMPTY;
      i=channel;
      channel=channelPrev;
      channelPrev=i;
      changeChannel = 1;
    }
    
    if(!strcmp(ir->command, DISPLAY)){
      ir->state=EMPTY;
      channelName[20]=0;
      sprintf(buff, "%s @ %s", channelName, satList.sat[satList.current].name);
      showOSD(buff, videoDev);
      timeOSD=30;
    }

    if(!strcmp(ir->command, RIGHT)){
      ir->state=EMPTY;
      if(satList.current++==satList.max)
	satList.current=0;
      showOSD(satList.sat[satList.current].name, videoDev);
      chlist.init(satList.sat[satList.current].file);
      //printf("Reading channel list from %s\n", satList.sat[satList.current].file);
      rotateDish(satList.sat[satList.current].num);
      sleep(1);
      if(ir->state == EMPTY)
	sleep(2);
      channelPrev=0;
      channel=0;
      changeChannel = 1; 
    }

    if(!strcmp(ir->command, LEFT)){
      ir->state=EMPTY;
      if(--satList.current<0)
	satList.current=satList.max;
      showOSD(satList.sat[satList.current].name, videoDev);
      chlist.init(satList.sat[satList.current].file);
      //printf("Reading channel list from %s\n", satList.sat[satList.current].file);
      rotateDish(satList.sat[satList.current].num);
      sleep(1);
      if(ir->state == EMPTY)
	sleep(2);
      channelPrev=0;
      channel=0;
      changeChannel = 1; 
    }

    if(!strcmp(ir->command, RECORD)){
      ir->state=EMPTY;
      strcpy(buff, channelName);
      for(i=1; i<20; i++)
	if(buff[i] == 0x20)
	  buff[i]='_';
      sscanf(buff, " %*d___%s", buff);
      for(i=0; i<20; i++)
	if(!((buff[i]>=48 && buff[i]<=57) || (buff[i]>=65 && buff[i]<=90) || 
		  (buff[i]>=97 && buff[i]<=122) ||  buff[i]==0))
	  buff[i]='_';
      buff[10]=0;
      gettimeofday(&tv, &tz);
      timef=localtime((time_t*)(&tv.tv_sec));
      drawOSD('r', videoDev);
      i=0; j=0;
      while(1){
	if(!(i++%(512*1024))){
	  j++;
	  if(j !=1 )
	    fclose(outFile);
	  sprintf(fileName, "/video/rec/%2.2d.%2.2d.%4.4d_%2.2d:%2.2d:%2.2d_%s_%1.1d.pva", 
		  timef->tm_mday, timef->tm_mon+1, timef->tm_year+1900, 
		  timef->tm_hour, timef->tm_min, timef->tm_sec, buff, j);
	  outFile=fopen(fileName, "w+"); 
	  if(outFile==NULL)
	    printf("can't open %s\n", fileName);
	  else
	    printf("start recording: %s\n", fileName);
	}
	read(videoDev, videoBuff, 2048);
	fwrite(videoBuff, 512, 4, outFile);
	if(ir->state != EMPTY){
	  if(!strcmp(ir->command, STOP)){
	  ir->state=0;
	  break;
	  }
	  ir->state=EMPTY;
	}
      }
      fclose(outFile);
      pmode.mode = VID_PLAY_RESET;
      ioctl(videoDev, VIDIOCSPLAYMODE, &pmode);

      drawOSD('s', videoDev);
      printf("stop recording\n");
      timeOSD=30;
      ir->state=EMPTY;
    }

    if(!strcmp(ir->command, PLAY)){
      closeOSD(videoDev);
      if(!getFileName(videoDev, fileName, directory))
	continue;
      file=open(fileName, O_RDONLY);
      if(file<0){
	fprintf(stderr, "can't open %s\n", fileName);
	continue;
      }
      drawOSD('p', videoDev);
      while(1){
	i=read(file, videoBuff, 2048);
	if(i>0)
	  write(videoDev, videoBuff, i);
	else{
	  if(!next_file(fileName, &file, 1))
	    break;
	} 
	if(ir->state == COMMAND){
	  if(!strcmp(ir->command, PAUSE)){
	    ir->state=EMPTY;
	    pmode.mode = VID_PLAY_PAUSE;
	    ioctl(videoDev, VIDIOCSPLAYMODE, &pmode); 
	    while(ir->state==EMPTY)
	      usleep(100000);
	  }
	  else if(!strcmp(ir->command, FASTFORWARD)){
	    ir->state=EMPTY;
	    pmode.mode = VID_PLAY_FAST_FORWARD;
	    ioctl(videoDev, VIDIOCSPLAYMODE, &pmode); 
	  }
	  else if(!strcmp(ir->command, SLOW)){
	    ir->state=EMPTY;
	    pmode.mode = VID_PLAY_SLOW_MOTION;
	    ioctl(videoDev, VIDIOCSPLAYMODE, &pmode); 
	  }
	  else if(!strcmp(ir->command, PLAY)){
	    ir->state=EMPTY;
	    pmode.mode = VID_PLAY_NORMAL;
	    ioctl(videoDev, VIDIOCSPLAYMODE, &pmode); 
	  }
	  else if(!strcmp(ir->command, LEFT)){
	    ir->state=EMPTY;
	    pmode.mode = VID_PLAY_PAUSE;
	    ioctl(videoDev, VIDIOCSPLAYMODE, &pmode); 
	    file_seek(videoDev, fileName, &file, go_to(videoDev, -1));
	    drawOSD('p', videoDev);
	  }
	  else if(!strcmp(ir->command, RIGHT)){
	    ir->state=EMPTY;
	    pmode.mode = VID_PLAY_PAUSE;
	    ioctl(videoDev, VIDIOCSPLAYMODE, &pmode); 
	    file_seek(videoDev, fileName, &file, go_to(videoDev, 1));
	    drawOSD('p', videoDev);
	  }
	  else if(!strcmp(ir->command, STOP)){
	    ir->state=EMPTY;
	    break;
	  }
	  else{
	    ir->state=EMPTY;
	    pmode.mode = VID_PLAY_NORMAL;
	    ioctl(videoDev, VIDIOCSPLAYMODE, &pmode); 
	  }
	}
      }
      ir->state=EMPTY;
      close(file);
      pmode.mode = VID_PLAY_RESET;
      ioctl(videoDev, VIDIOCSPLAYMODE, &pmode);
      drawOSD('s', videoDev);
      timeOSD=30;
      ir->state=EMPTY;
    }
    
    
    if(changeChannel==1){
      channel_set(chlist(channel), channelName, videoDev);
      channelName[34]=0;
      timeChann=0;
      showOSD(channelName, videoDev);
      timeOSD=30;
      changeChannel = 0;  
    }
    if(ir->state == EMPTY)
      irLoop=0;
    else{
      if(irLoop++ > 10){
	ir->state = EMPTY;
	irLoop=0;
      }
    }
  }
  close(videoDev);
}
void showOSD(char *text, int dev){
  struct drawcmd dc;

  OSDOpen(dev, 60, 10, 660, 56, 2, 0);
  OSDSetColor(dev, 1, 0, 0, 0, 128);
  OSDSetColor(dev, 2, 0, 0, 0, 128);
  OSDSetColor(dev, 3, 255, 255, 255, 128);
  OSDFill(dev, 1);

  dc.cmd=OSD_Text;
  dc.x0=0;
  dc.y0=7;
  dc.x1=3; 
  dc.data=text;
  dc.color=3+(2<<16);
  ioctl(dev, VIDIOCSOSDCOMMAND, &dc);
}

void closeOSD(int dev){
  struct drawcmd dc;
  dc.cmd=OSD_Close;
  ioctl(dev, VIDIOCSOSDCOMMAND, &dc);
}

void channel_set(Channel &ch, char *buf,int dev){
  struct frontend front;
  uint freq;

  sprintf(buf, " %d  %s", ch.Num(), ch.Name());
  //printf("Setting channel: %s\n", buf);
  ioctl(dev, VIDIOCGFRONTEND, &front);
  freq=(uint)ch.Freq()*1000UL;
  if (freq<11800000UL)
    front.freq=(freq -  9750000UL)*1000UL;
  else
    front.freq=(freq - 10600000UL)*1000UL; 
  front.ttk=(freq<11800000UL) ? 0 : 1;
  front.srate=ch.SR()*1000;
  front.fec=ch.FEC();
  front.video_pid=ch.V_pid();
  front.audio_pid=ch.A_pid();
  front.volt=ch.Pol()-9;
  front.diseqc=ch.Type()-10;
  ioctl(dev, VIDIOCSFRONTEND, &front);
}

void rotateDish(int num){
  unsigned int i;
  char buf[8];
  struct sockaddr_un addr;
  int fd;
  char buffer[32];
  
  usleep(300000);
  addr.sun_family=AF_UNIX;
  strcpy(addr.sun_path, LIRCD);
  fd=socket(AF_UNIX,SOCK_STREAM,0);
  connect(fd,(struct sockaddr *)&addr, sizeof(addr));
  //printf("Sending to analog receiver: %d\n", num);
  sprintf(buf, "%d", num);
  for(i=0; i<strlen(buf); i++){
    sprintf(buffer,"SEND_ONCE ANALOG %c\n", buf[i]);
    write(fd, (void *)buffer, strlen(buffer));
    usleep(300000);
  }
  close(fd);
}

int readSatList(struct satelliteList *satList){

  FILE* fp;
  int i,j;
  char path[64];
  char tmp[64];

  strcpy(path, getenv("HOME"));
  sprintf(tmp, "%s/.remote/satconf", path);
  fp=fopen(tmp, "r");
  if(fp==NULL){
    fprintf(stderr, "could not open %s\n", tmp);
    return 0;
  }
  j=0;
  i=-1;
  while(j != -1){
    i++;
    fscanf(fp,"%s",satList->sat[i].name);
    if(satList->sat[i].name[0]=='*'){
      fscanf(fp,"%s",satList->sat[i].name);
      satList->current=i-1;
    }
    fscanf(fp,"%s", tmp);
    sprintf(satList->sat[i].file, "%s/.remote/%s", path, tmp);
    j=fscanf(fp,"%d",&satList->sat[i].num);
  }
  fclose(fp);
  satList->max=i-1;
  return 1;
}

int getFileName(int dev, char *fileName, char *dir){

#define  LINES_NO    15       
#define  FONT_SIZE    1     
#define  HEIGHT      26  


  int i, k;
  char stmp[512];
  char btmp[512];
  char type[128];
  char name[128][64];
  char dirstrc[16][32];
  int dirnum;
  int fnum;
  FILE *p; 
  int start, size, cur;
  
  k=0;
  i=1;
  dirnum=0;
  while(dir[i] != 0){
    if(dir[i] == '/'){
      dirstrc[dirnum][k]=0;
      dirnum++;
      k=0;
      i++;
    }
    else{
      dirstrc[dirnum][k]=dir[i];
      i++;
      k++;
    }
  }
  dirstrc[dirnum][k]=0;
  if(i>1)
    dirnum++;
  
  while(1){
    start=0;
    size=LINES_NO;
    cur=0;
    stmp[0]=0;
    if(dirnum == 0)
      sprintf(stmp,"/");
    else for(i=0; i < dirnum; i++)
      sprintf(stmp,"%s/%s", stmp, dirstrc[i]);
    sprintf(btmp,"ls -lt %s | grep r", stmp);
    strcpy(dir, stmp);
    p = popen(btmp, "r");
    i=0;
    if (p) {
      while(fgets(btmp, 512, p) && sscanf(btmp, "%s %*s %*s %*s %*s %*s %*s %*s %s", 
					  stmp, btmp) == 2){
	sscanf(stmp, "%c", &type[i]);
	sscanf(btmp, "%s", &name[i][0]);
	i++;
	if(i >= 128)
	  break;
      }
      fnum = i;
      pclose(p);
    }
    else
      fprintf(stderr, "Error while opening pipe!");
    size=LINES_NO;
    if(fnum<LINES_NO)
      size=fnum;
  Replot:
    OSDClose(dev);
    OSDOpen(dev, 100, 30, 620, 456, 2, 0);
    OSDSetColor(dev, 0, 0, 0, 0, 160);
    OSDSetColor(dev, 1, 0, 255, 255, 255);
    OSDSetColor(dev, 2, 255, 0, 255, 255);
    OSDSetColor(dev, 3, 255, 255, 0, 255);
    //OSDFill(dev, 0);
    OSDText(dev, 50, 7, FONT_SIZE, 1, " ../");
    sprintf(stmp,"directory:  %s", dir);
    OSDText(dev, 120, 7, FONT_SIZE, 2, stmp);
    for(i=0; i<size; i++){
      sprintf(btmp, " %s", name[start+i]);
      if(type[start+i] == 'd')
	OSDText(dev, 50, 7+HEIGHT*(i+1), FONT_SIZE, 1, btmp);
      else
	OSDText(dev, 50, 7+HEIGHT*(i+1), FONT_SIZE, 3, btmp);
    }
    if(fnum == 0)
      cur=-1;
    i=cur;
    if(type[i+start] == 'd' || i==-1)
      OSDText(dev, 0, 7+HEIGHT*(i+1), FONT_SIZE, 1, " >>");
    else
      OSDText(dev, 0, 7+HEIGHT*(i+1), FONT_SIZE, 3, " >>");
    while(1){
      while(ir->state == EMPTY)
	usleep(50000);
      ir->state=EMPTY;
      //printf("lirc: remote: %s ir->command: %s\n", remote, ir->command);
      if(!strcmp(ir->command, ENTER)){
	if(i == -1){
	  if(dirnum > 0)
	    dirnum--;
	}
	else if(type[i+start] == 'd'){
	  dirnum++;
	  sprintf(dirstrc[dirnum-1], "%s", name[i+start]);
	}
	else{
	  stmp[0]=0;
	  for(k=0; k < dirnum; k++)
	    sprintf(stmp,"%s/%s", stmp, dirstrc[k]);
	  sprintf(stmp,"%s/%s", stmp, name[i+start]);
	  //printf("%s\n", stmp);
	  OSDClose(dev);
	  sprintf(fileName, "%s", stmp);
	  return 1;
	}
	OSDClose(dev);
	break;
      }
      else if(!strcmp(ir->command, UP) || !strcmp(ir->command, LEFT)){
	OSDText(dev, 0, 7+HEIGHT*(i+1), FONT_SIZE, 0, " >>");
	if(!strcmp(ir->command, LEFT))
	  i-=LINES_NO-1;
	i--;
	if(i<-1){
	  if(start == 0)
	    i=-1;
	  else{
	    start-=LINES_NO-1;
	    cur=LINES_NO-2;
	    if(start<0){
	      cur=start;
	      start=0;
	      cur+=LINES_NO-2;
	    }
	    goto Replot;
	  }
	}
	if(type[i+start] == 'd' || i == -1)
	  OSDText(dev, 0, 7+HEIGHT*(i+1), FONT_SIZE, 1, " >>");
	else
	  OSDText(dev, 0, 7+HEIGHT*(i+1), FONT_SIZE, 3, " >>");
      }
      else if(!strcmp(ir->command, DOWN) || !strcmp(ir->command, RIGHT)){
	OSDText(dev, 0, 7+HEIGHT*(i+1), FONT_SIZE, 0, " >>");
	if(!strcmp(ir->command, RIGHT))
	  i+=LINES_NO-1;
	i++;
	if(i>=size){
	  if(start+size == fnum)
	    i=size-1;
	  else{
	    start+=LINES_NO-1;
	    cur=1;
	    if(start>fnum-size){
	      cur=start;
	      start=fnum-size;
	      cur-=start-1;
	    }
	    goto Replot;
	  }
	}
	if(type[i+start] == 'd' || i == -1)
	  OSDText(dev, 0, 7+HEIGHT*(i+1), FONT_SIZE, 1, " >>");
	else
	  OSDText(dev, 0, 7+HEIGHT*(i+1), FONT_SIZE, 3, " >>");
      }
      else if(!strcmp(ir->command, MENU) || !strcmp(ir->command, NORMAL)){
	OSDClose(dev);
	return 0;
      }
    }
  }
}

void drawOSD(char sim, int dev){

  int i,j;
  unsigned char bmp[400];

  if(sim == 'r'){
    for(i=0; i<400; i++)
      bmp[i]=0x03;
  }
  if(sim == 's'){
    for(i=0; i<400; i++)
      bmp[i]=0x01;
  }
  if(sim == 'p'){
    for(i=0; i<20; i++)
      for(j=0; j<20; j++)
	if(abs(i-10)<int((20-j)/2))
	  bmp[20*i+j]=0x01;
	else
	  bmp[20*i+j]=0x00;
  }
 
  OSDClose(dev);
  OSDOpen(dev, 60, 30, 79, 49, 2, 0);
  OSDSetColor(dev, 0, 0, 0, 0, 0);
  OSDSetColor(dev, 1, 200, 200, 200, 200);
  OSDSetColor(dev, 2, 0, 0, 0, 0);
  OSDSetColor(dev, 3, 255, 0, 0, 255);
  OSDFill(dev, 0);
  OSDSetBlock(dev, 0, 0, 19, 19, 20, bmp);

}

int next_file(char *file_name, int *file, int next_file){
  
  int len=strlen(file_name);
  int f;
  
  if(next_file){
    if(!(file_name[len-5]>='1' && file_name[len-5]<='9' && len>=12))
      return 0;
    file_name[len-5]+=next_file;
    if(!(file_name[len-5]>='1' && file_name[len-5]<='9' && len>=12)){
      file_name[len-5]-=next_file;
      return 0;
    }
    f=open(file_name, O_RDONLY);
    if(f<0){
      fprintf (stderr, "can't open %s\n", file_name);
      file_name[len-5]-=next_file;
      return 0;
    }
    else{
      fprintf (stderr, "opening %s\n", file_name);
      close(*file);
      *file=f;
      return 1;
    }
  }
  return 0;
}


int file_seek(int dev, char *file_name, int *file, int go_to){

  struct video_play_mode pmode;
  long long int i, end, cur, seek;

  seek=go_to;
  seek*=1024*1024;
  cur=lseek(*file, 0, SEEK_CUR);
  end=lseek(*file, 0, SEEK_END);
  while(seek){
    i=cur+seek;
    //printf("cur: %d end:%d i: %d\n", (int)(cur/(1024*1024)), (int)(end/(1024*1024)), (int)(i/(1024*1024)));
    if(!(i<0 || i>end)){
      lseek(*file, i, SEEK_SET);
      seek=0;
    }
    else if(i<0){
      if(next_file(file_name, file, -1)){
	seek+=cur;
	cur=end=lseek(*file, 0, SEEK_END);
      }
      else{
	lseek(*file, 0, SEEK_SET);
	break;
      }
    }
    else{
      if(next_file(file_name, file, 1)){
	seek-=end-cur;
	cur=0;
	end=lseek(*file, 0, SEEK_END);
      }
      else{
	lseek(*file, -2*1024*1024, SEEK_END);
	break;
      }
    }
  }
  pmode.mode = VID_PLAY_RESET;
  ioctl(dev, VIDIOCSPLAYMODE, &pmode);
  if(seek==0)
    return 1;
  else
    return 0;
}

int go_to(int dev, int direction){

  int i=0;
  char tbuff[16];

  OSDClose(dev);
  OSDOpen(dev, 100, 40, 420, 80, 2, 0);
  OSDSetColor(dev, 0, 0, 0, 0, 255);
  OSDSetColor(dev, 1, 0, 255, 255, 255);
  OSDSetColor(dev, 2, 255, 0, 255, 255);
  if(direction==1)
    OSDText(dev, 20, 7, FONT_SIZE, 1, " FORWARD");
  else
    OSDText(dev, 20, 7, FONT_SIZE, 1, "BACKWARD");
  OSDText(dev, 170, 7, FONT_SIZE, 2, "  0");
  OSDText(dev, 240, 7, FONT_SIZE, 1, "MB");
  while(1){
    while(ir->state == EMPTY)
      usleep(50000);
    if(ir->state == NUM_KEY){
      i=10*i;
      i+=ir->number;
      if(i>1024*8){
	i=1024*8;
      }
      sprintf(tbuff, "%4d", i);
      OSDText(dev, 170, 7, FONT_SIZE, 2, tbuff);
      if(i==1024*8)
	break;
    }
    else if(!strcmp(ir->command, ENTER))
      break;
    else if(!strcmp(ir->command, NORMAL)){
      i=0;
      break;
    }
    ir->state=EMPTY;
  }
  direction*=i;
  //printf("size: %d\n", direction);
  sleep(1);
  OSDClose(dev);
  return direction;
}
